package eu.jonahbauer.chat.bot.api;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.time.LocalDateTime;


@JsonTypeInfo(
        use = JsonTypeInfo.Id.NAME,
        property = "type"
)
@JsonSubTypes({
        @JsonSubTypes.Type(value = Message.Ping.class, name = "ping"),
        @JsonSubTypes.Type(value = Message.Pong.class, name = "pong"),
        @JsonSubTypes.Type(value = Message.Ack.class, name = "ack"),
        @JsonSubTypes.Type(value = Message.Post.class, name = "post")
})
public sealed interface Message {
    Ping PING = new Ping();

    record Ping() implements Message { }

    record Pong() implements Message { }

    record Ack() implements Message { }

    record Post(
            long id,
            @NotNull String name,
            @NotNull String message,
            @NotNull String channel,
            @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd HH:mm:ss")
            @NotNull LocalDateTime date,
            @Nullable Long delay,
            @JsonProperty("user_id")
            @Nullable Long userId,
            @JsonProperty("username")
            @Nullable String userName,
            @NotNull String color,
            int bottag
    ) implements Message {
        public boolean bot() {
            return bottag != 0;
        }
    }
}
