package eu.jonahbauer.chat.server.management.impl;

import eu.jonahbauer.chat.bot.config.BotConfig;
import eu.jonahbauer.chat.server.bot.ChatBotSupervisor;
import eu.jonahbauer.chat.server.management.BotConfigSupport;
import eu.jonahbauer.chat.server.management.ChatBotSupportMXBean;
import lombok.Getter;
import lombok.extern.log4j.Log4j2;

import java.lang.management.ManagementFactory;

@Getter
@Log4j2
public class ChatBotSupport implements ChatBotSupportMXBean {
    private final String name;
    private final BotConfigSupport config;

    public static void register(String name, BotConfig config) {
        try {
            var server = ManagementFactory.getPlatformMBeanServer();
            server.registerMBean(new AdvancedMBean(new ChatBotSupport(name, config)), ChatBotSupportMXBean.getObjectName(name));
        } catch (Exception ex) {
            log.error("Could not register bot as an MBean.", ex);
        }
    }

    public static void unregister(String name) {
        try {
            var server = ManagementFactory.getPlatformMBeanServer();
            server.unregisterMBean(ChatBotSupportMXBean.getObjectName(name));
        } catch (Exception ex) {
            log.error("Could not unregister bot as an MBean.", ex);
        }
    }

    private ChatBotSupport(String name, BotConfig config) {
        this.name = name;
        this.config = new BotConfigSupport(config);
    }

    public void stop() throws InterruptedException {
        ChatBotSupervisor.INSTANCE.stop(name);
    }
}
