package eu.jonahbauer.chat.server.management.impl;

import eu.jonahbauer.chat.server.management.SocketState;
import eu.jonahbauer.chat.server.management.SocketSupportMXBean;
import eu.jonahbauer.chat.server.socket.SocketSupervisor;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

import java.lang.management.ManagementFactory;
import java.util.Date;

@Log4j2
@Getter
@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
public class SocketSupport implements SocketSupportMXBean {
    private final String channel;

    public static void register(String channel) {
        try {
            var server = ManagementFactory.getPlatformMBeanServer();
            server.registerMBean(new AdvancedMBean(new SocketSupport(channel)), SocketSupportMXBean.getObjectName(channel));
        } catch (Exception ex) {
            log.error("Could not register socket as an MBean.", ex);
        }
    }

    public static void unregister(String name) {
        try {
            var server = ManagementFactory.getPlatformMBeanServer();
            server.unregisterMBean(SocketSupportMXBean.getObjectName(name));
        } catch (Exception ex) {
            log.error("Could not unregister socket as an MBean.", ex);
        }
    }

    @Override
    public Date getCooldownUntil() {
        var cooldown = SocketSupervisor.INSTANCE.getCooldownUntil(channel);
        return cooldown != null ? Date.from(cooldown) : null;
    }

    @Override
    public SocketState getState() {
        return SocketSupervisor.INSTANCE.getState(channel);
    }


    @Override
    public void stop() throws InterruptedException {
        SocketSupervisor.INSTANCE.stop(channel);
    }

    @Override
    public void restart() {
        SocketSupervisor.INSTANCE.restart(channel);
    }

    @Override
    public void send(String name, String message, boolean bottag, boolean publicid) {
        SocketSupervisor.INSTANCE.send(channel, name, message, bottag, publicid);
    }
}
